{% load i18n %}

<div class="col1-2">
  <div class="alert alert-info text-center">
    {% blocktrans %}
    Your comment is in moderation.<br/>
    It has to be reviewed before being published.<br/>
    Thank you for your patience and understanding.
    {% endblocktrans %}
  </div>
</div>
