{% load i18n %}

<div class="col1-2">
  <div class="alert alert-info text-center">
    {% translate "Comment confirmation requested" %}
  </div>
  <p>
    {% blocktrans %}A confirmation message has been sent to your email address. Please, click on the link in the message to confirm your comment.{% endblocktrans %}
  </p>
</div>
