{% load i18n %}
{% load comments_ink %}

<div id="top-comment-pagination" class="inline-centered pagination">
  <span class="step-links">
    {% if page_obj.has_previous %}
      <a href="?{% render_qs_params page 1 %}#comments">{% trans "First" %}</a>
      <a href="?{% render_qs_params page page_obj.previous_page_number %}#comments">&laquo; {% trans "Previous" %}</a>
    {% endif %}

    <span class="current">
      {% blocktrans with page_number=page_obj.number num_pages=page_obj.paginator.num_pages %}Comments page {{ page_number }} of {{ num_pages }}{% endblocktrans %}
    </span>

    {% if page_obj.has_next %}
      <a href="?{% render_qs_params page page_obj.next_page_number %}#comments">{% trans "Next" %} &raquo;</a>
      <a href="?{% render_qs_params page page_obj.paginator.num_pages %}#comments">{% trans "Last" %}</a>
    {% endif %}
  </span>
</div>
