{% load i18n %}
{% load comments_ink %}

<div
  id="reply-to-{{ comment.id }}"
  class="{% if not is_input_allowed or comment.is_removed %}not-{% endif %}reply-box-img {{ comment.level|reply_css_thread_range }}"
>
  {% if is_input_allowed and not comment.is_removed %}
    {{ comment.level|indent_divs }}
    {{ comment.level|hline_div }}
    <div class="content">
      <section class="reply-form" data-dci="reply-form">
        <form id="reply-form-{{ comment.id }}" method="post" action="{% url 'comments-ink-reply' comment.id %}">{% csrf_token %}
          <input type="hidden" name="reply_to" value="{{ comment.id }}" />
          {% if is_paginated %}
          <input type="hidden" name="{{ comments_page_qs_param }}" value="{{ page_obj.number }}" />
          <input type="hidden" name="{{ comments_fold_qs_param }}" value="{{ cfold }}" />
          {% endif %}
          <div class="col1-2">
            <div class="inline-left">
              <button type="submit" class="secondary small" data-dci="reply-button">{% translate "Write a reply" %}</button>
            </div>
          </div>
        </form>
      </section>
    </div>
  {% endif %}
</div>
